<?php
/**
*
* @package phpBB3
* @version $Id: stopforumspam.php,v 1.0.1 2013/05/13
* @copyright (c)Rich McGirr (aka RMcGirr83 @ phpBB.com)
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

function checkStopForumSpam($username, $email, $ip)
{
	global $config, $user, $db, $auth, $phpbb_root_path, $phpEx;
	
	if (empty($config['sfs_enabled']))
	{
		return;
	}
	$user->add_lang('mods/stopforumspam');
	// we need to urlencode for spaces
	$username = urlencode(stripslashes($username));
	// Default value
	$spam_score = 0;
	$sfs_api_key = !empty($config['sfs_api_key']) ? $config['sfs_api_key'] : false;

	$sfs_send_email	= $config['sfs_send_email'];
	$sfs_log_admin_message = $config['sfs_log_admin_message'];

	/* Threshold score to reject registration and/or guest posting */
	$sfs_threshold_score = $config['sfs_threshold_score'];	
	
	// Query the SFS database and pull the data into script
	if ($email)
	{
		$xmlUrl = 'http://www.stopforumspam.com/api?username='.$username.'&ip='.$ip.'&email='.$email.'&f=xmldom';
	}
	else
	{
		$xmlUrl = 'http://www.stopforumspam.com/api?username='.$username.'&ip='.$ip.'&f=xmldom';
	}
	$xmlStr = get_file($xmlUrl);

	// Check if user is a spammer, but only if we successfully got the SFS data
	if ($xmlStr)
	{
		$xmlObj = simplexml_load_string($xmlStr);		
	   
		//break ouf of the check if not returned as true
		if ($xmlObj->success != true)
		{
			// free up some memory
			unset($xmlObj);
			return false;
		}
		// Assign points for the total number of times each have been flagged
		$ck_username = $xmlObj->username->frequency;
		$ck_email = $xmlObj->email->frequency;
		$ck_ip = $xmlObj->ip->frequency;
		
		// Let's not ban a registrant with a common username, who is otherwise clean
		if ($ck_email + $ck_ip == 0)
		{
			$ck_username = 0;
		}
		
		// Let's not ban a registrant with a common IP address, who is otherwise clean
		if ($ck_username + $ck_email == 0)
		{
			$ck_ip = 0;
		}
		
		// Return the total score
		$spam_score = ($ck_username + $ck_email + $ck_ip);
		// If we've got a spammer we'll take away their soup!
		if ($spam_score >= $sfs_threshold_score)
		{			
			//deencode the stuffs
			$username = urldecode($username);			
			if (!empty($sfs_api_key) && !empty($email))
			{
				// add the spammer to the SFS database
				$http_request = 'http://www.stopforumspam.com/add.php';
				$http_request .= '?username=' . $username;
				$http_request .= '&ip_addr=' . $ip;
				$http_request .= '&email=' . $email;
				$http_request .= '&api_key=' . $sfs_api_key;

				$response = get_file($http_request);
				
				if (!$response)
				{
					add_log('admin', 'LOG_SFS_DOWN', gmstrftime("%A %d-%b-%y %T %Z", time()));
				}
				elseif ($sfs_log_admin_message)
				{
					// successful
					$sfs_ip_check = sprintf($user->lang['SFS_IP_STOPPED'], $ip);
					$sfs_username_check = sprintf($user->lang['SFS_USERNAME_STOPPED'], $username);
					$sfs_email_check = !empty($email) ? sprintf($user->lang['SFS_EMAIL_STOPPED'], $email) : $user->lang['SFS_POSTING'];
					
					add_log('admin', 'LOG_SFS_SUBMITTED', $sfs_username_check, $sfs_email_check, $sfs_ip_check);
				}
			}		
			// Do we need to send an email? If so, do it
			if ($sfs_send_email)
			{
				if (!class_exists('messenger'))
				{
					include($phpbb_root_path . 'includes/functions_messenger.' . $phpEx);
				}
				
				$messenger = new messenger(true);
				// Email headers
				$messenger->anti_abuse_headers($config, $user);				
				
				// get admins to send an email to
				$admin_ary = $auth->acl_get_list(false, 'a_', false);
				$admin_ary = (!empty($admin_ary[0]['a_'])) ? $admin_ary[0]['a_'] : array();
				
				// Also include founders
				$where_sql = ' WHERE user_type = ' . USER_FOUNDER;				
				if (sizeof($admin_ary))
				{
					$where_sql .= ' OR ' . $db->sql_in_set('user_id', $admin_ary);
				}
				$sql = 'SELECT user_id, username, user_email, user_lang, user_jabber, user_notify_type
					FROM ' . USERS_TABLE . ' ' .
					$where_sql;
				$result = $db->sql_query($sql);
								
				// Loop through our list of users
				while ($row = $db->sql_fetchrow($result))
				{			
					$messenger->template('stopforumspam', $row['user_lang']);
					$messenger->to($row['user_email'], $row['username']);
					$messenger->im($row['user_jabber'], $row['username']);
					
					$messenger->assign_vars(array(
						'ADM_USERNAME'	=> htmlspecialchars_decode($row['username']),
						'SITENAME'		=> htmlspecialchars_decode($config['sitename']),
						'USER_IP'		=> $ip,
						'USERNAME'		=> htmlspecialchars_decode($username),
						'USER_EMAIL'	=> !empty($email) ? htmlspecialchars_decode($email) : $user->lang['SFS_POSTING'],
						'SFS_SCORE'		=> $spam_score,
					));
	
					$messenger->send($row['user_notify_type']);
				}
				$db->sql_freeresult($result);				
			}
			
			// Do we need to log an administrator message?
			if ($sfs_log_admin_message)
			{
				$sfs_ip_check = sprintf($user->lang['SFS_IP_STOPPED'], $ip);
				$sfs_username_check = sprintf($user->lang['SFS_USERNAME_STOPPED'], $username);
				$sfs_email_check = !empty($email) ? sprintf($user->lang['SFS_EMAIL_STOPPED'], $email) : $user->lang['SFS_POSTING'];
				
				add_log('admin', 'LOG_SFS_MESSAGE', $sfs_username_check, $sfs_email_check, $sfs_ip_check);
			}
			return sprintf($user->lang['NO_SOUP_FOR_YOU'], $config['board_contact']);
		}
		else
		{
			return false;
		}
	}
	else
	{
		if ($email)
		{
			add_log('admin', 'LOG_SFS_ERROR_MESSAGE_ADMIN_POST', $username, $ip);
		}
		else
		{
			add_log('admin', 'LOG_SFS_ERROR_MESSAGE_ADMIN_REG', $username, $email, $ip);
		}
		return ('SFS_ERROR_MESSAGE');
	}
}


// Try to get with file_get_contents, else use curl
function get_file($url)
{
	if (ini_get('allow_url_fopen')) // file_get_contents() will work
	{
		// set a timeout
		$timeout = stream_context_create(array('http'=>
			array(
				'timeout' => 5 // 5 sec
			)
		));	
		return file_get_contents($url, false, $timeout);
	}
	else if (function_exists('curl_init')) // We'll use curl
	{
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_TIMEOUT, 5);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
        $contents = curl_exec($ch);
        curl_close($ch);
		// if nothing is returned (SFS is down)
		if(empty($contents))
		{
			return false;
		}
        return $contents;
	}
}